package edu.washington.cs.oneswarm.f2f.share;

import java.io.UnsupportedEncodingException;
import java.util.Map;
import java.util.logging.Logger;

import org.bouncycastle.util.encoders.Base64;
import org.gudy.azureus2.core3.torrent.TOTorrent;
import org.gudy.azureus2.core3.torrent.TOTorrentFile;
import org.gudy.azureus2.core3.torrent.impl.TOTorrentFileImpl;
import org.gudy.azureus2.core3.torrent.impl.TOTorrentImpl;
import org.gudy.azureus2.core3.util.Base32;

import edu.washington.cs.oneswarm.f2f.FileListFile;

public class ShareManagerTools
{
	private static Logger logger = Logger.getLogger(ShareManagerTools.class.getName());

	//	public static void setSha1AndEd2kHash(TOTorrent t, int fileIndex,
	//			FileListFile f) throws TOTorrentException {
	//
	//		
	//		//Map torrentMap = (Map) t.serialiseToMap().get("info");
	//		
	//		if (t.isSimpleTorrent()) {
	//			TOTorrentFileImpl t.getFiles()[].get
	//			setSha1AndEd2k(file);
	//		} else {
	//			TOTorrentFileImpl t.getFiles()[fileIndex].get
	//			Object filePropertiesObj = torrentMap.get("files");
	//
	//			if (filePropertiesObj == null || !(filePropertiesObj instanceof List)) {
	//				logger.finest("not adding sha1 to '" + f.getFileName()
	//						+ "', no files map");
	//				return;
	//			}
	//			List fileProperties = (List) filePropertiesObj;
	//
	//			if (fileProperties != null && fileProperties.size() > fileIndex) {
	//				Object pObj = fileProperties.get(fileIndex);
	//				if (pObj != null && pObj instanceof Map) {
	//					Map pMap = (Map) pObj;
	//					setSha1AndEd2k(pMap, f);
	//				} else {
	//					logger.finest("not adding sha1 hash to '" + f.getFileName()
	//							+ "', no map");
	//				}
	//			} else {
	//				logger.finest("not adding sha1 hash to '" + f.getFileName()
	//						+ "', no properties");
	//			}
	//		}
	//	}

	public static void setSha1AndEd2k(TOTorrent torrent,
			TOTorrentFile torrentFile, FileListFile f) {
		Map torrentMap = null;
		if (torrent.isSimpleTorrent()) {
			logger.finest("simple torrent: " + new String(torrent.getName()));
			try {
				if ((torrent instanceof TOTorrentImpl)) {
					torrentMap = ((TOTorrentImpl) torrent).getAdditionalInfoProperties();
				} else {
					torrentMap = (Map) torrent.serialiseToMap().get("info");
				}

			} catch (Exception e) {
				e.printStackTrace();
				return;
			}
		} else if (torrentFile instanceof TOTorrentFileImpl) {
			torrentMap = ((TOTorrentFileImpl) torrentFile).getAdditionalProperties();
		} else {
			return;
		}
		if (torrentMap.containsKey(FileListFile.KEY_SHA1_HASH)) {
			byte[] hash = (byte[]) torrentMap.get(FileListFile.KEY_SHA1_HASH);
			f.setSha1Hash(hash);
			logger.finest("adding sha1 hash to '" + f.getFileName() + "': "
					+ new String(Base64.encode(hash)));
		} else {
			logger.finest("not adding sha1 hash to '" + f.getFileName()
					+ "', no key in map");
		}
		if (torrentMap.containsKey(FileListFile.KEY_ED2K_HASH)) {
			byte[] hash = (byte[]) torrentMap.get(FileListFile.KEY_ED2K_HASH);
			f.setEd2kHash(hash);
			logger.finest("adding ed2k hash to '" + f.getFileName() + "': "
					+ new String(Base64.encode(hash)));
		} else {
			logger.finest("not adding ed2k hash to '" + f.getFileName()
					+ "', no key in map");
		}
	}

	public static String baseXtoBase64(String encoded, int expectedBytes)
			throws UnsupportedEncodingException {
		return new String(Base64.encode(baseXdecode(encoded, expectedBytes)));
	}

	public static byte[] baseXdecode(String encoded, int expectedBytes)
			throws UnsupportedEncodingException {
		// check what encoding that is likely
		encoded = encoded.trim();
		int stringLength = encoded.length();
		logger.finest("trying to find base of '" + encoded + "' input bytes="
				+ stringLength + ", expected after decode=" + expectedBytes);
		/*
		 * check for base 64, base64 converts 3 input bytes to 4 output bytes
		 */
		// base64 has special rules for padding, if the string ends with "=" the expected length will be 1 more
		int base64expectedBytes = expectedBytes;
		if (encoded.endsWith("=")) {
			base64expectedBytes = expectedBytes + 1;
		}
		if (stringLength * 3 == base64expectedBytes * 4) {
			logger.finest("detected base64 encoding");
			try {
				byte[] result = Base64.decode(encoded);
				return result;
			} catch (Exception e) {
				logger.finest("tried base 64 encoding but got exception: "
						+ e.getMessage());
			}
		}
		/*
		 * base32 encodes in chunks of 40 bits, each 40 bit chunk becomes 8 characters
		 */
		if (stringLength * 5 == expectedBytes * 8) {
			logger.finest("detected base32 encoding");
			try {
				byte[] result = Base32.decode(encoded);
				return result;
			} catch (Exception e) {
				logger.finest("tried base 32 encoding but got exception: "
						+ e.getMessage());
			}
		}
		/*
		 * base16 encoding (hex)
		 */
		if (stringLength == 2 * expectedBytes) {
			logger.finest("detected base16 encoding");
			try {
				byte[] result = base16Decode(encoded);
				return result;
			} catch (Exception e) {
				logger.finest("tried base 16 encoding but got exception: "
						+ e.getMessage());
			}
		}

		throw new UnsupportedEncodingException("unable to detect base for input '"
				+ encoded + "'");
	}

	private static byte[] base16Decode(String enc) {
		enc = enc.trim();
		int l = enc.length();
		int i = 0, j = 0;
		byte[] dec = new byte[l / 2];

		while (i < l) {
			byte n, b;

			b = (byte) (enc.charAt(i));
			if ((b >= (byte) ('0')) && (b <= (byte) ('9')))
				n = (byte) (b - (byte) ('0'));
			else
				n = (byte) (b - (byte) ('A') + 10);

			b = (byte) (enc.charAt(i + 1));
			if ((b >= (byte) ('0')) && (b <= (byte) ('9')))
				n = (byte) ((n << 4) + b - (byte) ('0'));
			else
				n = (byte) ((n << 4) + b - (byte) ('A') + 10);

			dec[j] = n;

			i += 2;
			j++;
		}
		return dec;
	}

	private static final byte[] HEX_CHAR_TABLE = {
		(byte) '0',
		(byte) '1',
		(byte) '2',
		(byte) '3',
		(byte) '4',
		(byte) '5',
		(byte) '6',
		(byte) '7',
		(byte) '8',
		(byte) '9',
		(byte) 'a',
		(byte) 'b',
		(byte) 'c',
		(byte) 'd',
		(byte) 'e',
		(byte) 'f'
																						 };

	public static String base16Encode(byte[] raw) {
		byte[] hex = new byte[2 * raw.length];
		int index = 0;

		for (byte b : raw) {
			int v = b & 0xFF;
			hex[index++] = HEX_CHAR_TABLE[v >>> 4];
			hex[index++] = HEX_CHAR_TABLE[v & 0xF];
		}
		try {
			return new String(hex, "ASCII").toUpperCase();
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "";
		}
	}

}
