package edu.washington.cs.oneswarm.ui.gwt.client.newui;

import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.ChangeListener;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.HorizontalSplitPanel;
import com.google.gwt.user.client.ui.Hyperlink;
import com.google.gwt.user.client.ui.KeyboardListener;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

import edu.washington.cs.oneswarm.ui.gwt.client.OneSwarmDialogBox;
import edu.washington.cs.oneswarm.ui.gwt.client.OneSwarmGWT;
import edu.washington.cs.oneswarm.ui.gwt.client.OneSwarmRPCClient;
import edu.washington.cs.oneswarm.ui.gwt.client.ReportableErrorDialogBox;
import edu.washington.cs.oneswarm.ui.gwt.client.Updateable;
import edu.washington.cs.oneswarm.ui.gwt.rpc.FriendInfoLite;
import edu.washington.cs.oneswarm.ui.gwt.rpc.SerialChatMessage;
import edu.washington.cs.oneswarm.ui.gwt.rpc.TorrentInfo;
import edu.washington.cs.oneswarm.ui.gwt.rpc.UserOfflineException;

public class ChatDialog extends OneSwarmDialogBox implements Updateable {

	public static final int WIDTH = 540;
	public static final int HEIGHT = 350;

	public static final int MAX_LENGTH = 1024;

	String[] keys_by_nick = null;
	Map<Integer, String> listEntryToKey = new HashMap<Integer, String>();
	Map<String, String> keyToNick = new HashMap<String, String>();
	Map<String, FriendInfoLite> keyToFriend = new HashMap<String, FriendInfoLite>();
	private String mSelectedBase64PublicKey;
	private VerticalPanel mChatPanel = new VerticalPanel();
	private ScrollPanel mChatScroll = new ScrollPanel();
	final private HelpButton mErrorHelpButton = new HelpButton("Some text here");
	final private VerticalPanel mMainRHSVP = new VerticalPanel();
	private HorizontalSplitPanel mainPanel;
	// private Button mSendButton = new Button("Send");
	private final TextBox mTextBox = new TextBox();
	private boolean mSending;
	private FriendInfoLite[] mFriends;

	/**
	 * When switching among multiple chats, we should keep the display constant
	 * among them
	 */
	private Map<String, VerticalPanel> mKeyToChatPanel = new HashMap<String, VerticalPanel>();
	private Set<String> mShowingFullHistory = new HashSet<String>();
	private EntireUIRoot mUIRoot;

	private static ChatDialog showing = null;

	public static boolean showing() {
		return showing != null;
	}

	public static boolean tryHide() {
		if (showing != null) {
			if (showing.isWriting()) {

				return false;
			} else {
				showing.hide();
				return true;
			}
		}
		return true;
	}

	public ChatDialog(FriendInfoLite[] friendInfoLites, String inSelectedPublicKey, EntireUIRoot root) {
		setText("Chat");

		mFriends = friendInfoLites;
		mUIRoot = root;

		for (FriendInfoLite f : mFriends) {
			keyToFriend.put(f.getPublicKey(), f);
		}

		mTextBox.setMaxLength(MAX_LENGTH);

		mSelectedBase64PublicKey = inSelectedPublicKey;

		mainPanel = new HorizontalSplitPanel();

		createLeftWidget();
		mainPanel.setLeftWidget(mUserList);

		createChatPanel(false);
		mChatScroll.setWidget(mChatPanel);
		mChatScroll.setHeight((HEIGHT - (39 + 16 + 3)) + "px");
		mChatScroll.setAlwaysShowScrollBars(false);

		HorizontalPanel hp = new HorizontalPanel();
		mTextBox.setFocus(true);

		mTextBox.addKeyboardListener(new KeyboardListener() {
			public void onKeyDown(Widget sender, char keyCode, int modifiers) {
			}

			public void onKeyPress(Widget sender, char keyCode, int modifiers) {
				if (keyCode == 13) { // enter
					sendCurrentMessage();
				}
			}

			public void onKeyUp(Widget sender, char keyCode, int modifiers) {
			}
		});

		hp.add(mTextBox);
		hp.setSpacing(3);
		mTextBox.setWidth("96%");
		hp.setWidth("100%");

		mMainRHSVP.add(mChatScroll);
		HorizontalPanel linkAndHelp = new HorizontalPanel();
		linkAndHelp.add(mErrorHelpButton);
		mErrorHelpButton.setVisible(false);
		final Hyperlink linkSwarms = new Hyperlink("Link to selected swarms", "#");
		linkAndHelp.add(linkSwarms);
		linkAndHelp.setSpacing(3);
		linkSwarms.addClickListener(new ClickListener() {

			public void onClick(Widget sender) {
				TorrentInfo[] selected = mUIRoot.getSelectedSwarms();
				if (selected == null) {
					return;
				}

				if (mTextBox.isEnabled() == false) {
					return;
				}

				boolean limited_friend = false, f2f_only = false;
				if (keyToFriend.get(mSelectedBase64PublicKey) != null) {
					if (keyToFriend.get(mSelectedBase64PublicKey).isCanSeeFileList() == false) {
						limited_friend = true;
					}
				}

				/**
				 * To compensate for the myriad of representations we have of a
				 * hash. We need to do this on the backend. This is extremely
				 * frustrating to me.
				 */
				String[] tids = new String[selected.length];
				for (int i = 0; i < tids.length; i++) {
					tids[i] = selected[i].getTorrentID();
				}
				OneSwarmRPCClient.getService().getBase64HashesForOneSwarmHashes(OneSwarmRPCClient.getSessionID(), tids, new AsyncCallback<String[]>() {
					public void onFailure(Throwable caught) {
					}

					public void onSuccess(String[] result) {
						if (result == null) {
							System.err.println("null result on base64 conversion");
							return;
						}

						for (int i = 0; i < result.length; i++) {
							String id = "id:" + result[i];
							mTextBox.setText(mTextBox.getText() + id + (i < result.length - 1 ? " " : ""));
							if (mTextBox.getText().length() + id.length() > MAX_LENGTH) {
								break;
							}
						}
					}
				});

				for (TorrentInfo t : selected) {
					if (t.isF2FOnly()) {
						f2f_only = true;
					}
				}

				if (limited_friend || f2f_only) {
					StringBuilder errorString = new StringBuilder();
					if (limited_friend) {
						errorString.append("This friend does not have access to your file list, and so may not be able to access links to your files. ");
					}
					if (f2f_only) {
						errorString.append("Your friend may not have permission to access the selected files of your friends.");
					}
					mErrorHelpButton.setText(errorString.toString());
					mErrorHelpButton.setVisible(true);
				} else {
					mErrorHelpButton.setVisible(false);
				}
			}
		});
		mMainRHSVP.add(linkAndHelp);
		mMainRHSVP.setCellHorizontalAlignment(linkAndHelp, HorizontalPanel.ALIGN_RIGHT);
		mMainRHSVP.add(hp);
		mMainRHSVP.setCellWidth(hp, "100%");
		mMainRHSVP.setWidth("100%");
		mainPanel.setRightWidget(mMainRHSVP);
		mainPanel.setSplitPosition("25%");

		mainPanel.setWidth(WIDTH + "px");
		mainPanel.setHeight(HEIGHT + "px");

		this.setWidget(mainPanel);

		OneSwarmGWT.addToUpdateTask(this);
	}

	public boolean isWriting() {
		return mTextBox.getText().length() > 0;
	}

	public void onDetach() {
		super.onDetach();
		OneSwarmGWT.removeFromUpdateTask(this);
		showing = null;
	}

	public void onAttach() {
		super.onAttach();
		showing = this;
	}

	private void sendCurrentMessage() {
		if (mSending) {
			return;
		}

		mSending = true;
		// mSendButton.setEnabled(false);
		String messageText = mTextBox.getText().trim();

		if (messageText.length() == 0) {
			mTextBox.setText("");
			return;
		}

		SerialChatMessage serialChatMessage = new SerialChatMessage();
		serialChatMessage.setMessage(messageText);

		OneSwarmRPCClient.getService().sendChatMessage(OneSwarmRPCClient.getSessionID(), mSelectedBase64PublicKey, serialChatMessage, new AsyncCallback<Void>() {
			public void onSuccess(Void result) {
				mSending = false;
				nextRPC = 0; // induce immediate refresh
			}

			public void onFailure(Throwable caught) {
				try {
					throw caught;
				} catch (UserOfflineException e) {
					Window.alert("Can't send message - user is offline");
				} catch (Throwable e) {
					new ReportableErrorDialogBox(caught.getMessage(), false);
				}
				mTextBox.setEnabled(false);
			}
		});

		mTextBox.setText("");
	}

	private void createChatPanel(final boolean include_read) {

		mLastAdded = WhichAdding.First;
		mChatScroll.remove(mChatPanel);

		if (mSelectedBase64PublicKey == null) {
			return;
		}

		boolean force = false;
		if (include_read && !mShowingFullHistory.contains(mSelectedBase64PublicKey)) {
			force = true;
		}

		if (mKeyToChatPanel.containsKey(mSelectedBase64PublicKey) && !force) {
			mChatPanel = mKeyToChatPanel.get(mSelectedBase64PublicKey);
			mChatScroll.setWidget(mChatPanel);
			mChatScroll.scrollToBottom();
			return;
		} else {
			mChatPanel = new VerticalPanel();
			mChatPanel.setWidth("100%");
			mKeyToChatPanel.put(mSelectedBase64PublicKey, mChatPanel);
		}

		if (include_read) {
			mShowingFullHistory.add(mSelectedBase64PublicKey);
		}

		mTextBox.setEnabled(true);

		mChatScroll.setWidget(mChatPanel);

		mChatPanel.add(new Label("Loading..."));

		OneSwarmRPCClient.getService().getMessagesForUser(OneSwarmRPCClient.getSessionID(), mSelectedBase64PublicKey, include_read, 0, new AsyncCallback<SerialChatMessage[]>() {
			public void onFailure(Throwable caught) {
				caught.printStackTrace();
			}

			public void onSuccess(final SerialChatMessage[] initialResult) {
				mChatPanel.clear();

				Hyperlink show = null;
				if (!include_read) {
					show = new Hyperlink("Show all previous messages", "");
					show.addClickListener(new ClickListener() {
						public void onClick(Widget sender) {
							createChatPanel(true);
							mChatScroll.setWidget(mChatPanel);
						}
					});
				} else {
					show = new Hyperlink("Clear log", "");
					show.addClickListener(new ClickListener() {
						public void onClick(Widget sender) {
							final String key_shadow = mSelectedBase64PublicKey;
							System.out.println("clearing for: " + mSelectedBase64PublicKey + " / " + keyToNick.get(mSelectedBase64PublicKey));
							OneSwarmRPCClient.getService().clearChatLog(OneSwarmRPCClient.getSessionID(), key_shadow, new AsyncCallback<Integer>() {
								public void onFailure(Throwable caught) {
									caught.printStackTrace();
								}

								public void onSuccess(Integer result) {
									System.out.println("cleared " + result);
									/**
									 * Or else we'll just put up this panel
									 * again
									 */
									mKeyToChatPanel.remove(key_shadow);
									createChatPanel(true);
								}
							});
						}
					});
				}

				mChatPanel.add(show);
				mChatPanel.setCellWidth(show, "100%");
				mChatPanel.setCellHorizontalAlignment(show, HorizontalPanel.ALIGN_CENTER);

				if (initialResult.length > 4) {
					addMessagesToChatPanel(initialResult);
				} else if (include_read == false) {
					// add a little history.
					OneSwarmRPCClient.getService().getMessagesForUser(OneSwarmRPCClient.getSessionID(), mSelectedBase64PublicKey, true, 4, new AsyncCallback<SerialChatMessage[]>() {
						public void onFailure(Throwable caught) {
							caught.printStackTrace();
						}

						public void onSuccess(SerialChatMessage[] result) {
							/**
							 * If we've re-requested some messages which were
							 * unread before, mark them again as such now.
							 */
							for (SerialChatMessage neu : result) {
								for (SerialChatMessage old : initialResult) {
									if (old.getUid() == neu.getUid()) {
										neu.setUnread(old.isUnread());
									}
								}
							}

							addMessagesToChatPanel(result, true);
						}
					});
				}
			}
		});
	}

	protected void addMessagesToChatPanel(SerialChatMessage[] result) {
		addMessagesToChatPanel(result, false);
	}

	private enum WhichAdding {
		First, Incoming, Outgoing
	};

	WhichAdding mLastAdded = WhichAdding.First;

	protected void addMessagesToChatPanel(SerialChatMessage[] result, boolean old) {
		for (SerialChatMessage chat : result) {
			HorizontalPanel hp = new HorizontalPanel();

			Label chatLabel = new Label(chat.getMessage());
			Label timestamp = new Label();

			chatLabel.setWordWrap(true);

			transmorphLinks(chatLabel, "http", "<a href=\"#\" onclick=\"window.open('", "', '_blank', '')\">", "</a>");
			transmorphLinks(chatLabel, "id:", "<a href=\"#" + EntireUIRoot.SEARCH_HISTORY_TOKEN, "\">", "</a>");
			transmorphLinks(chatLabel, "oneswarm:", "<a href=\"", "\">", "</a>");

			Date now = new Date();
			Date then = new Date(chat.getTimestamp());

			String stampStr;
			if ((now.getTime() - chat.getTimestamp()) < 86400000 && now.getDate() == then.getDate()) { // 1
				// day
				stampStr = then.getHours() + ":" + (then.getMinutes() < 10 ? "0" + then.getMinutes() : then.getMinutes());
			} else {
				stampStr = (then.getMonth() + 1) + "/" + then.getDate();
			}
			timestamp.setText(stampStr);

			if (old && !(chat.isUnread())) {
				DOM.setStyleAttribute(chatLabel.getElement(), "color", "grey");
				System.out.println(chat.getMessage() + " unread: " + chat.isUnread());
			}

			DOM.setStyleAttribute(timestamp.getElement(), "color", "grey");
			DOM.setStyleAttribute(timestamp.getElement(), "fontSize", "80%");

			if (chat.isOutgoing()) {
				// transition, print our name
				if (mLastAdded.equals(WhichAdding.Incoming) || mLastAdded.equals(WhichAdding.First)) {
					HTML html = new HTML("<b>" + chat.getNickname() + "</b>");
					mChatPanel.add(html);
					// if( old && chat.isUnread() == false ) {
					DOM.setStyleAttribute(html.getElement(), "color", "grey");
					// }
					mChatPanel.setCellHorizontalAlignment(html, HorizontalPanel.ALIGN_RIGHT);
				}

				mLastAdded = WhichAdding.Outgoing;
				hp.add(chatLabel);
				hp.add(timestamp);
				hp.setCellHorizontalAlignment(timestamp, HorizontalPanel.ALIGN_RIGHT);
				mChatPanel.add(hp);
				mChatPanel.setCellHorizontalAlignment(hp, HorizontalPanel.ALIGN_RIGHT);
			} else {
				// transition, print their name
				if (mLastAdded.equals(WhichAdding.Outgoing) || mLastAdded.equals(WhichAdding.First)) {
					HTML html = new HTML("<b>" + chat.getNickname() + "</b>");
					mChatPanel.add(html);
					// if( old && chat.isUnread() == false ) {
					DOM.setStyleAttribute(html.getElement(), "color", "grey");
					// }
					mChatPanel.setCellHorizontalAlignment(html, HorizontalPanel.ALIGN_LEFT);
				}

				mLastAdded = WhichAdding.Incoming;
				hp.add(timestamp);
				hp.setCellHorizontalAlignment(timestamp, HorizontalPanel.ALIGN_LEFT);
				hp.add(chatLabel);
				mChatPanel.add(hp);
				mChatPanel.setCellHorizontalAlignment(hp, HorizontalPanel.ALIGN_LEFT);
			}
			hp.setCellVerticalAlignment(timestamp, VerticalPanel.ALIGN_BOTTOM);
			hp.setCellVerticalAlignment(hp, VerticalPanel.ALIGN_BOTTOM);
			hp.setSpacing(2);
			hp.setCellWidth(timestamp, "35px");
		}

		mChatScroll.scrollToBottom();
	}

	private void transmorphLinks(Label chatLabel, String tag, String prefix, String suffix, String close) {
		try {
			String innerHTML = chatLabel.getElement().getInnerHTML();
			System.out.println("inner html is: " + innerHTML);
			StringBuilder out = new StringBuilder();
			int curr = 0;
			boolean done = false;
			while (!done) {
				curr = innerHTML.indexOf(tag, curr);
				if (curr == -1) {
					done = true;
					break;
				}

				out.append(innerHTML.substring(0, curr) + prefix);
				int end = innerHTML.indexOf(' ', curr);
				if (end == -1) {
					end = innerHTML.length();
				}
				out.append(innerHTML.subSequence(curr, end) + suffix + innerHTML.subSequence(curr, end) + close);
				curr = out.length();
				out.append(innerHTML.subSequence(end, innerHTML.length()));
				innerHTML = out.toString();
			}
			// System.out.println("transmorphed: " + innerHTML);
			chatLabel.getElement().setInnerHTML(innerHTML);
		} catch (Exception e) {
			// eat exception, do nothing to chat label itself.
			e.printStackTrace();
		}
	}

	final ListBox mUserList = new ListBox();
	final ChangeListener userChangeListener = new ChangeListener() {
		public void onChange(Widget sender) {
			System.out.println("change listener " + mUserList.getSelectedIndex() + " / ");

			if (mUserList.getSelectedIndex() == -1) {
				mSelectedBase64PublicKey = null;
				createChatPanel(false);
				ChatDialog.this.setText("Chat");
				return;
			}

			mSelectedBase64PublicKey = keys_by_nick[mUserList.getSelectedIndex()];
			ChatDialog.this.setText("Chat: " + keyToNick.get(mSelectedBase64PublicKey));
			mUserList.setItemText(mUserList.getSelectedIndex(), keyToNick.get(mSelectedBase64PublicKey));

			System.out.println(mSelectedBase64PublicKey);

			createChatPanel(false);
		}
	};

	private void createLeftWidget() {

		mUserList.setVisibleItemCount(10);
		mUserList.setHeight(HEIGHT + "px");
		mUserList.setWidth("100%");

		mUserList.addChangeListener(userChangeListener);

		mUserList.addItem("Loading...");
		OneSwarmRPCClient.getService().getUsersWithMessages(OneSwarmRPCClient.getSessionID(), new AsyncCallback<Map<String, String[]>>() {
			public void onFailure(Throwable caught) {
				caught.printStackTrace();
			}

			public void onSuccess(final Map<String, String[]> result) {
				mUserList.clear();

				/**
				 * Result is base64Key -> String[]{name, unread messages}
				 */

				/**
				 * Build up the list of users with which we 1) have chatted
				 * (i.e., with history) or 2) could chat (those that have chat
				 * capability). We get 1) from the friend list passed to the
				 * constructor and 2) from an RPC to the backend DB.
				 */
				java.util.Set<String> userSet = new HashSet<String>();
				userSet.addAll(result.keySet());
				for (FriendInfoLite f : mFriends) {
					if (f.isSupportsChat() && f.isAllowChat()) {
						userSet.add(f.getPublicKey());
					}
				}

				for (String key : userSet) {
					if (result.containsKey(key)) {
						keyToNick.put(key, result.get(key)[0]);
					} else {
						for (FriendInfoLite f : mFriends) {
							if (f.getPublicKey().equals(key)) {
								keyToNick.put(key, f.getName());
							}
						}
					}
				}

				keys_by_nick = keyToNick.keySet().toArray(new String[0]);
				Arrays.sort(keys_by_nick, new Comparator<String>() {
					public int compare(String o1, String o2) {
						return keyToNick.get(o1).compareTo(keyToNick.get(o2));
					}
				});

				mUserList.setVisibleItemCount(Math.max(keys_by_nick.length, 10));

				boolean picked = false;
				for (String key : keys_by_nick) {
					String unreadStr = "";
					if (result.containsKey(key)) {
						if (result.get(key)[1].equals("0") == false) {
							unreadStr = " - " + result.get(key)[1];
						}
					}
					mUserList.addItem(keyToNick.get(key) + unreadStr);

					if (mSelectedBase64PublicKey != null) {
						if (mSelectedBase64PublicKey.equals(key)) {
							mUserList.setSelectedIndex(mUserList.getItemCount() - 1);
							ChatDialog.this.setText("Chat: " + keyToNick.get(mSelectedBase64PublicKey));
							mUserList.setItemText(mUserList.getSelectedIndex(), keyToNick.get(mSelectedBase64PublicKey));
						}
					} else if (unreadStr.equals("") == false && !picked) {
						/**
						 * If nothing selected, at least prefer something that
						 * has unread messages
						 */
						picked = true;
						mUserList.setSelectedIndex(mUserList.getItemCount() - 1);
						userChangeListener.onChange(mUserList);
					}
				}
			}
		}); // getUsersWithMessages RPC

	}

	long nextRPC = 0;

	public void update(int count) {
		if (nextRPC < System.currentTimeMillis()) {
			nextRPC = System.currentTimeMillis() + 5 * 1000;
			OneSwarmRPCClient.getService().getMessagesForUser(OneSwarmRPCClient.getSessionID(), mSelectedBase64PublicKey, false, 0, new AsyncCallback<SerialChatMessage[]>() {
				public void onFailure(Throwable caught) {
					caught.printStackTrace();
				}

				public void onSuccess(SerialChatMessage[] result) {
					if (result.length > 0) {
						addMessagesToChatPanel(result);
					}
					nextRPC = System.currentTimeMillis() + 1000;
				}
			});
		}
	}
}
