/*
 * Created on 6 sept. 2003
 *
 */
package org.gudy.azureus2.irc;

import org.jibble.pircbot.Colors;
import org.jibble.pircbot.PircBot;
import org.jibble.pircbot.User;

import org.gudy.azureus2.core3.util.Constants;
import org.gudy.azureus2.core3.util.FileUtil;
import org.gudy.azureus2.plugins.*;

import java.net.*;
import java.io.*;
import java.util.Arrays;
import java.util.Locale;

import com.aelitis.azureus.core.AzureusCoreFactory;
import com.aelitis.azureus.core.dht.DHT;
import com.aelitis.azureus.plugins.dht.DHTPlugin;

import org.gudy.azureus2.plugins.network.ConnectionManager;
import org.gudy.azureus2.plugins.utils.*;


/**
 * @author Olivier
 * 
 */

public class 
IrcClient 
	extends PircBot 
{
	public static final String	CONFIG_IRC_SERVER 			= "Irc Server";
	public static final String	CONFIG_IRC_SERVER_DEFAULT	= "irc.freenode.net";
	
	public static final String	CONFIG_IRC_CHANNEL			= "Irc Channel";
	public static final String	CONFIG_IRC_CHANNEL_DEFAULT	= "";
	
	public static final String	CONFIG_IRC_USER				= "Irc Login";
	public static final String	CONFIG_IRC_USER_DEFAULT		= "";
	
	public static final String	CONFIG_IRC_SEND_USER_INFO				= "Irc User Info";
	public static final boolean	CONFIG_IRC_SEND_USER_INFO_DEFAULT		= true;
	
	public static final String	CONFIG_IRC_LOG				= "Irc Log";
	public static final boolean	CONFIG_IRC_LOG_DEFAULT		= false;
	
	private PluginInterface		plugin_interface;
	private PluginConfig		plugin_config;
	private PluginManager		plugin_manager;
	private LocaleUtilities 	locale_utils;
	
  private String srvName;
  private String channel;
  private String azVersion;
  //private String[] badWords;
  private String dHTStatus;
  private IrcListener listener;
  private String userName;
  private String static_id;
  private ConnectionManager	connection_manager; 
   
  public 
  IrcClient(
  	PluginInterface	_plugin_interface,
	IrcListener 	_listener)
  {
	  plugin_interface	= _plugin_interface;
	  listener = _listener;
	  //badWords = new String[500];
  	
	  locale_utils = plugin_interface.getUtilities().getLocaleUtilities();
	  
	  plugin_config = plugin_interface.getPluginconfig();
  	  	
	  plugin_manager = plugin_interface.getPluginManager();
  	    	
	  azVersion = plugin_interface.getAzureusVersion();

  	
  	srvName 	= plugin_config.getPluginStringParameter(CONFIG_IRC_SERVER, CONFIG_IRC_SERVER_DEFAULT );
  	channel 	= plugin_config.getPluginStringParameter(CONFIG_IRC_CHANNEL, CONFIG_IRC_CHANNEL_DEFAULT );
  	try {
  		channel = channel.replaceAll(" ", "");
  	} catch (Exception Ex) {
  	}
  	userName 	= plugin_config.getPluginStringParameter(CONFIG_IRC_USER, CONFIG_IRC_USER_DEFAULT);
  	
  	if(userName.equals("")) {
  		listener.systemMessage(locale_utils.getLocalisedMessageText("IrcClient.noNick"));
  		return;
  	}
  	
  	if(channel.equals("")) {
  		channel = locale_utils.getLocalisedMessageText("IrcClient.defaultChannel");
  	}
    
  	static_id = plugin_config.getStringParameter( "ID", "        " ).substring( 0, 8 );  //first chars of the client instance id

    String az_version = azVersion;
    az_version = az_version.replaceAll("_", "" );
    az_version = az_version.replaceAll("\\.", "" );
    PluginManager	pm = AzureusCoreFactory.getSingleton().getPluginManager();
    connection_manager = pm.getDefaultPluginInterface().getConnectionManager();
    
    setName( userName ); //also sets NICK
    setLogin( static_id );
    setVersion( "AZ" +az_version+ "|" +static_id+ "|" +getVersion() );
    
    
    Thread t = new Thread() {
      /* (non-Javadoc)
       * @see java.lang.Thread#run()
       */
      public void run() {
        try {
        	/*try {
        		URL badWordAddress = new URL("http://Tanyvska.co.uk/Words.txt");
        		URLConnection badConnection = badWordAddress.openConnection();
        		DataInputStream dis = new DataInputStream(badConnection.getInputStream());
        		String inputLine;
        		int wordtoInsert = 0;
                while ((inputLine = dis.readLine()) != null) {                	
                	badWords[wordtoInsert] = inputLine;
                	wordtoInsert = wordtoInsert + 1;
                }
        	}
        	catch (Exception Ex) {
        	}*/
        	listener.systemMessage("");  
        	listener.systemMessage(locale_utils.getLocalisedMessageText("IrcClient.copyright"));
        	listener.systemMessage("");
        	listener.systemMessage(locale_utils.getLocalisedMessageText("IrcClient.connecting") + " " + srvName);
        	connect(srvName);
        	listener.systemMessage(locale_utils.getLocalisedMessageText("IrcClient.connected") + " " + srvName);
        	listener.systemMessage(locale_utils.getLocalisedMessageText("IrcClient.joining") + " " + channel);
        	joinChannel(channel);
        	listener.systemMessage(locale_utils.getLocalisedMessageText("IrcClient.channel") + " " + channel + " " +  locale_utils.getLocalisedMessageText("IrcClient.joined"));
        	}
        catch (Exception e) {
        	//e.printStackTrace();
        	listener.systemMessage(locale_utils.getLocalisedMessageText("IrcClient.error") + " : " + e.getMessage());
        	}
        }
      };
      t.setDaemon(true);
      t.start();
  }
  protected void onMessage(String channel, String sender, String login, String hostname, String message) {
    if(listener != null) {
      listener.messageReceived(sender,Colors.removeFormattingAndColors(message));
    }
  }
  
  public void close() {    
    super.quitServer( plugin_interface.getAzureusName() + " " + plugin_interface.getAzureusVersion());
    try {
      super.dispose();
    } catch(Exception e) {
      
    }
  }
  
  public void sendMessage(String message) {
	  if ((message.startsWith("!")) || (message.startsWith("@"))){
		  listener.notice("WARNING", "Do not use ! and @ triggers here, there are NO files - read the topic and http://azureus.aelitis.com/wiki/index.php/Rules_for_IRC");
	  } else {
		  /* TODO MjrTom's stuff
		  boolean sent = false;
		  String messageS = "";
		  String[] words =  message.split(" ");
		  if (sent == false) {
			  for (int i = 0; i < words.length; i++) {
				  for (int h = 0; h < badWords.length; h++) {
					  if (! badWords[h].equalsIgnoreCase(words[i])) {
						  if (sent == false) {*/
							  super.sendMessage(channel,message);
							  listener.messageReceived(userName,message);
							  /*sent = true;
							  }
						  }
					  else {
						  listener.systemMessage("boo");
						  }
					  }
				  }
			  }*/
		  }
	  }
  
  
  protected void onJoin(String channel, String sender, String login, String hostname) {
    listener.systemMessage(sender + " " + locale_utils.getLocalisedMessageText("IrcClient.hasjoined")  + (sender.equals(userName)?" " + channel:""));
    listener.clientEntered(sender);
  }
  
  protected void onKick(String channel, String kickerNick, String kickerLogin, String kickerHostname, String recipientNick, String reason) {
    listener.systemMessage(kickerNick + " " + locale_utils.getLocalisedMessageText("IrcClient.haskicked") + " " + recipientNick + " (" + reason + ").");
    listener.clientExited(recipientNick);
	  if (recipientNick.equals(userName)){
		    listener.notice("WARNING", "You are no longer in the channel as you have been kicked (" + reason + ") Please respect the rules... http://azureus.aelitis.com/wiki/index.php/Rules_for_IRC");
		    listener.allExited();
		  }
  }
  
  protected void onQuit(String sourceNick, String sourceLogin, String sourceHostname, String reason) {
    listener.systemMessage(sourceNick + " " + locale_utils.getLocalisedMessageText("IrcClient.hasleft") + " (" + reason + ").");
    listener.clientExited(sourceNick);
  }
  
  protected void onPart(String channel, String sender, String login, String hostname)  {
    listener.systemMessage(sender + " " + locale_utils.getLocalisedMessageText("IrcClient.hasleft")  + (sender.equals(userName)?" " + channel:""));
    listener.clientExited(sender);
  }
  
  
  protected void onNickChange(String oldNick, String login, String hostname, String newNick) 
  {
    listener.systemMessage(oldNick + " " + locale_utils.getLocalisedMessageText("IrcClient.nowknown") + " " + newNick);
    listener.clientExited(oldNick);
    listener.clientEntered(newNick);
  }
  
  protected void onAction(String sender, String login, String hostname, String target, String action) 
  {
    listener.action(sender,action);
  }
  
  /**
   * @return
   */
  public String getUserName() {
    return userName;
  }

  /**
   * @param userName
   */
  public void setUserName(String _userName) {
	  if (! _userName.startsWith("`") && ! _userName.startsWith("") && ! _userName.startsWith("|") && ! _userName.startsWith("[") && ! _userName.startsWith("]") && ! _userName.startsWith("_") && ! _userName.startsWith("^")){
		    userName = _userName;
		    plugin_config.setPluginParameter(CONFIG_IRC_USER, userName);
		    changeNick(userName);		  
	  }
	  else{
		listener.notice("WARNING", "Your nick was not changed as it would have made responding to you more difficult, Please choose one containing only alphanumerc characters... http://azureus.aelitis.com/wiki/index.php/Rules_for_IRC");
	  }
  }
  
  public void sendAction(String action) {
    super.sendAction(channel,action);
  }
  
  protected void onUserList(String channel, User[] users) {
    if(! this.channel.equals(channel))
       return;
    for(int i = 0 ; i < users.length ;i++) {
      listener.clientEntered(users[i].getNick());
    }    
  }
  
  protected void onPrivateMessage(String sender, String login, String hostname, String message)
  {
    listener.privateMessage(sender,message);
  }
  
  protected void onNotice(String sourceNick, String sourceLogin, String sourceHostname, String target, String notice)
  {
	if ((! notice.contains("[Global Notice]")) && (! sourceNick.equals("NickServ")))
	{
		listener.notice(sourceNick,notice);
	}
  } 
  
  protected void onServerResponse(int code, String response)  {

    switch (code)
    {
      //No such nick name:
      case ERR_NOSUCHNICK:
	listener.systemMessage(response);
	break;

/*
      default:
	listener.privateMessage("Server: ", response);
*/
    }
  }

  /**
   * Tests if a given user is a channel op.
   * @param nick The user's nick.
   * @return true of the user is a channel op, false otherwise. 
   */
  public boolean isChanelOp(String nick)
  {
    User[] users = getUsers(channel);
    int i;

    for (i = 0; i < users.length; i++)
	if (users[i].getNick().equalsIgnoreCase(nick) && users[i].isOp())
	    return true;

    return false;
  }
  
  public boolean isChanelVoice(String nick)
  {
    User[] users = getUsers(channel);
    int i;

    for (i = 0; i < users.length; i++)
	if (users[i].getNick().equalsIgnoreCase(nick) && users[i].hasVoice())
	    return true;

    return false;
  }
  
  protected void onTopic(String channel, String topic, String setBy, long date, boolean changed) {
    listener.systemMessage(locale_utils.getLocalisedMessageText("IrcClient.topicforchannel") + " " + channel + " : " + topic);
  }
  
  protected void onDisconnect() {
    listener.systemMessage(locale_utils.getLocalisedMessageText("IrcClient.disconnected") + " " + srvName);
  }
  
  /**
   * @return
   */
  public String getChannel() {
    return channel;
  }

  /**
   * @return
   */
  public String getSrvName() {
    return srvName;
  }
  
  public void changeChannel(String channel, String message) {
	  try {
		  channel = channel.replaceAll(" ", "");
	  } catch (Exception Ex) {
		  
	  }  
	  if (! channel.equalsIgnoreCase(this.channel)) {
	    partChannel(this.channel + " :" + message);    
	    User[] users = super.getUsers(this.channel);
	    for(int i=0 ; i< users.length ; i++) {
	      listener.clientExited(users[i].getNick());
	    }
	    this.channel = channel;
	    //plugin_config.setPluginParameter(CONFIG_IRC_CHANNEL, this.channel);
	    joinChannel(this.channel);
	  } else {
		  listener.notice("Information", "You're already in that channel (" + channel +") unless you've been kicked, in which case you would have been notified");
		  }
	  }

/* (non-Javadoc)
 * @see org.jibble.pircbot.PircBot#onInfo(java.lang.String, java.lang.String, java.lang.String, java.lang.String)
 */
protected void onInfo(String sourceNick, String sourceLogin, String sourceHostname, String target) {
	try {
	String UPnPEnabled;
	String UserSelectedMode;
	String BindToIP = "N/A";
	PluginInterface dht_pi 	= AzureusCoreFactory.getSingleton().getPluginManager().getPluginInterfaceByClass( "com.aelitis.azureus.plugins.dht.DHTPlugin" );
	PluginInterface upnp_pi = AzureusCoreFactory.getSingleton().getPluginManager().getPluginInterfaceByClass( "com.aelitis.azureus.plugins.upnp.UPnPPlugin" );

	boolean bSendInfo = plugin_config.getPluginBooleanParameter(CONFIG_IRC_SEND_USER_INFO);
	if ((isChanelOp(sourceNick) == true) || (isChanelVoice(sourceNick) == true)) {
		
		if(bSendInfo){
			
			this.sendMessage(sourceNick, "Info for " + getName() + " (" + static_id +") in " + channel +" - " + azVersion + " installed in " +System.getProperty("user.dir") +" - " + System.getProperty("os.name") + " " + System.getProperty("sun.os.patch.level") + " " + System.getProperty("os.version") + " " + System.getProperty("os.arch") + "");
			if (System.getProperty("os.name").startsWith("Windows")){
	        	try {
	            String SLine = "";
	        	Process p = Runtime.getRuntime().exec("REG QUERY \"HKLM\\HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0\" /v ProcessorNameString");
	        	InputStream is = p.getInputStream();
	        	InputStreamReader isr = new InputStreamReader(is);
	        	BufferedReader br = new BufferedReader(isr);
	        	String line;
	        	while ((line = br.readLine()) != null) {
	           		if (line.contains("ProcessorNameString")) {
	           			SLine = "CPU: " + line.substring(line.indexOf("REG_SZ") + 6) + "";
	           			try {
	           			SLine = SLine.replaceAll("	              ","");
	           			SLine = SLine.replaceAll("	","");
	           			} catch (Exception Ex) {
	           			}
	        			} 
	        		} 
	        	br.close();
	        	this.sendMessage(sourceNick, "System: " + SLine);
	        	}
			    catch (Exception Ex) {
			    	
			    }
		    }			
			File dir = new File(System.getProperty("java.home") + File.separator + ".." + File.separator + (Constants.isOSX?".."+File.separator:""));
		    String[] children = dir.list();
		    String filedir = "";
		    if (children != null)
		    {
		        for (int i=0; i<children.length; i++)
		        {
		            filedir = children[i] + ", " + filedir;
		        }
		    }
		    String JLine = "";
	        if (System.getProperty("os.name").startsWith("Windows")){
	        	try {
	        	Process p = Runtime.getRuntime().exec("REG QUERY \"HKLM\\SOFTWARE\\JavaSoft\\Java Update\\Policy\\\" /s");
	        	InputStream is = p.getInputStream();
	        	InputStreamReader isr = new InputStreamReader(is);
	        	BufferedReader br = new BufferedReader(isr);
	        	String line;
	        	while ((line = br.readLine()) != null) {
	           		if (line.contains("EnableJavaUpdate")) {
	        			if (line.contains("0x0")) {
	        				JLine = " | Updater: Disabled | ";
	        				} else
	        				{
	        					JLine = " | Updater: Enabled | ";
	        					}
	        			} else if (line.contains("EnableAutoUpdateCheck")) {
	        				if (line.contains("0x0")) {
		        				JLine = JLine + "Auto update check: Disabled";
		        				} else
		        				{
		        					JLine = JLine + "Auto update check: Enabled";
		        					}
	        			}
	        		} 
	        	br.close();
	        	}
			    catch (Exception Ex) {
			    	
			    }
		    }
		    this.sendMessage(sourceNick, "Java: " + System.getProperty("java.version") + " installed in " + System.getProperty("java.home") + " | Installations: " + filedir + "" + JLine);
			if (! plugin_config.getStringParameter("Bind IP","").equals("")) {
				BindToIP = plugin_config.getStringParameter("Bind IP","");
			}
			String portUDP = "";
			if (plugin_config.getIntParameter("Plugin.DHT.dht.portdefault") == 1) {
				portUDP = plugin_config.getIntParameter("TCP.Listen.Port",6881) + "";
			} else {
				portUDP = plugin_config.getIntParameter("Plugin.DHT.dht.port") + "";
			}
		    this.sendMessage(sourceNick, "Connection: TCP Port: " + plugin_config.getIntParameter("TCP.Listen.Port",6881) + " | UDP Port: " + portUDP + " | Max connection attempts: " + plugin_config.getIntParameter("network.max.simultaneous.connect.attempts",8) + " | Bind to local IP: " + BindToIP + "");
		    String upSeedingOnly = "";
		    if (plugin_config.getIntParameter("enable.seedingonly.upload.rate",0) == 0)
		    {
		    	upSeedingOnly = "N/A";
		    }
		    else
		    {
		    	upSeedingOnly = "" + plugin_config.getIntParameter("Max Upload Speed Seeding KBs", 0);
		    }
		    this.sendMessage(sourceNick, "Transfer: Max upload speed: " + plugin_config.getIntParameter("Max Upload Speed KBs", 0) + " | Max upload speed while seeding: " + upSeedingOnly + " | Max download speed: " + plugin_config.getIntParameter("Max Download Speed KBs", 0) + " | Default max upload slots: " + plugin_config.getIntParameter("Max Uploads", 4) + "");
		    this.sendMessage(sourceNick, "Transfer: Max connections per torrent: " + plugin_config.getIntParameter("Max.Peer.Connections.Per.Torrent", 80) + " | Max connections globally: " + plugin_config.getIntParameter("Max.Peer.Connections.Total", 400) + "");
		    String numberSeedingOnly = "";
		    if (plugin_config.getIntParameter("StartStopManager_bMaxActiveTorrentsWhenSeedingEnabled", 0) == 0)
		    {
		    	numberSeedingOnly = "N/A";
		    }
		    else
		    {
		    	numberSeedingOnly = "" + plugin_config.getIntParameter("MaxActiveTorrentsWhenSeeding", 0);
		    }
		    this.sendMessage(sourceNick, "Queue: Max downloads: " + plugin_config.getIntParameter("max downloads", 4) + " | Max torrents: " + plugin_config.getIntParameter("max active torrents", 4) + " | Max torrents while seeding: " + numberSeedingOnly + " | Ignore Ratio: " +  plugin_config.getFloatParameter("Stop Ratio") + ":1");
		    if ( dht_pi != null ){
		    	//Code I don't fully understand below... Damn my copy/paste-ing...
		    	DHTPlugin	dhtPlugin = (DHTPlugin)dht_pi.getPlugin();	
		    	long dht_count = -1;
		    	String dHTStatusR = "";
		    	switch (dhtPlugin.getStatus()) {
				case DHTPlugin.STATUS_RUNNING:
					DHT[] dhts = dhtPlugin.getDHTs();
						boolean reachable = dhts.length > 0	&& dhts[0].getTransport().isReachable();
						if (reachable) {
							if (dhts.length == 0)
								//And this? what is the point of dht_count -1 means unconnectable? and 0? or >0?
								dht_count = 0;
							else
								dht_count = dhts[0].getControl().getStats()
										.getEstimatedDHTSize();
						} else {
							dht_count = -1;
						}
					if (dht_count == -1) {
						dHTStatusR = "(Unreachable)";
					} else {
						dHTStatusR = "(Reachable)";
					}
					if (dht_count == 0) {
						dHTStatus = "Running " + dHTStatusR + " Users: Unknown";
					} else {
						dHTStatus = "Running " + dHTStatusR + " Users: "  + dhts[0].getControl().getStats().getEstimatedDHTSize();
					}
					break;

				case DHTPlugin.STATUS_DISABLED:
					dHTStatus = "Disabled";
					break;

				case DHTPlugin.STATUS_INITALISING:
					dHTStatus = "Initialising";
					break;

				case DHTPlugin.STATUS_FAILED:
					dHTStatus = "Failed";
					break;
					}
		    }/*
			    if(dhtPlugin == null || dhtPlugin.getStatus() == DHTPlugin.STATUS_DISABLED ){
			    	dHTStatus = "Disabled";
			    }
			    else if(dhtPlugin.getStatus() == DHTPlugin.STATUS_FAILED ) {
			    	dHTStatus = "Failed";
			    }
			    else if( dhtPlugin.getStatus() == DHTPlugin.STATUS_INITALISING ) {
			    	dHTStatus = "Initialising";
			    }
			    else if( dhtPlugin.getStatus() == DHTPlugin.STATUS_RUNNING ) {
			    	DHT[]	dhts = dhtPlugin.getDHTs();
			    	String dHTReach = "";
			  		if ( dhts.length == 0 || dhts[0].getTransport().isReachable()){
			    	dHTReach = "Reachable";
		    		}
			  		else
			  		{
			  			dHTReach = "Unreachable";
			    	}
			    	dHTStatus = "Running (" + dHTReach + ") Users: " + numberFormat.format(dhts[0].getControl().getStats().getEstimatedDHTSize());
			    }
			    else{
			    	dHTStatus = "Unknown";
			    }
		    }
		    else{
		    	dHTStatus = "Unavailable";
		    }*/
	        int	nat_status = connection_manager.getNATStatus();
	        String nat_State = "";
	        switch (nat_status) {
			case ConnectionManager.NAT_UNKNOWN:
				nat_State = "Grey";
				break;
			case ConnectionManager.NAT_OK:
				nat_State = "Green";
				break;
			case ConnectionManager.NAT_PROBABLY_OK:
				nat_State = "Yellow";
				break;
			default:
				nat_State = "Red";
				break;
				}
		    this.sendMessage(sourceNick, "Status: DHT: " + dHTStatus + " | NAT: " + nat_State + "");
	        String eLine = "";
	        if (System.getProperty("os.name").startsWith("Windows")) {
	        	Process p = Runtime.getRuntime().exec("ipconfig /all");
	        	InputStream is = p.getInputStream();
	        	InputStreamReader isr = new InputStreamReader(is);
	        	BufferedReader br = new BufferedReader(isr);
	        	String line;	        	
	        	while ((line = br.readLine()) != null) {
	        		if (line.contains("IP Address")) {
					line = line.substring(line.indexOf("IP Address"));
					eLine = eLine + "IP Address: " + line.substring(line.indexOf(": ") + 2) + " | ";
				} else if (line.contains("Subnet Mask")) {
					line = line.substring(line.indexOf("Subnet Mask"));
					eLine = eLine + "Subnet Mask: " + line.substring(line.indexOf(": ") + 2) + " | ";
				} else if (line.contains("Default Gateway")) {
					line = line.substring(line.indexOf("Default Gateway"));
					eLine = eLine + "Default Gateway: " + line.substring(line.indexOf(": ") + 2) + " | ";
				} else if (line.contains("DNS Servers")) {
					line = line.substring(line.indexOf("DNS Servers"));
					eLine = eLine + "DNS Server: " + line.substring(line.indexOf(": ") + 2) + " | ";
				}
	        	} 
			br.close();
	        }
	        else
			{
	        	try {
	        		String address = InetAddress.getLocalHost ().getHostAddress();
					//String address = InetAddress.getLocalHost().getHostName();
					  this.sendMessage(sourceNick, "IP: " + address);
				}
				catch (UnknownHostException ex) {
					  this.sendMessage(sourceNick, "IP: IP Address could not be resolved");
				}
	        }
			this.sendMessage(sourceNick, "Network: " + eLine);
			String languageM = "";
		    if (Locale.getDefault().toString().equals(""))
		    {
		    	languageM = "en_US";
		    }
		    else
		    {
		    	languageM = Locale.getDefault().toString();
		    }
		    
		    String FLine = "XP Firewall: Unknown";
	        if (System.getProperty("os.name").equals("Windows XP") && System.getProperty("sun.os.patch.level").contains("2")){
	        	Process p = Runtime.getRuntime().exec("REG QUERY HKLM\\System\\CurrentControlSet\\Services\\SharedAccess\\Parameters\\FirewallPolicy\\StandardProfile /v EnableFirewall");
	        	InputStream is = p.getInputStream();
	        	InputStreamReader isr = new InputStreamReader(is);
	        	BufferedReader br = new BufferedReader(isr);
	        	String line;
	        	while ((line = br.readLine()) != null) {
	           		if (line.contains("REG_DWORD")) {
	           			if (line.contains("0x0")) {
	           				FLine = "XP Firewall: Disabled | ";	  
	           				} else {
	           					FLine = "XP Firewall: Enabled | ";
	        					}
	           			}
	        		} 
	        	br.close();
	        	}
			if (plugin_config.getIntParameter("Plugin.UPnP.upnp.enable", 1) == 0) {
				UPnPEnabled = "Disabled";
			}else{
				String info = "";
				
				if ( upnp_pi != null){
					info = " (" + upnp_pi.getPluginconfig().getPluginStringParameter( "plugin.info" ) + ")";
				} else {
					info = " (Unable to determine router)";
				}
				UPnPEnabled = "Enabled" + info; 				
			}
			if (plugin_config.getIntParameter("User Mode", 0) == 1) {
				UserSelectedMode = "Intermediate";
				}
			else if (plugin_config.getIntParameter("User Mode", 0) == 2){
				UserSelectedMode = "Advanced";
				}
			else {
				UserSelectedMode = "Beginner";
				}       
		    this.sendMessage(sourceNick, "Misc: UPnP: " + UPnPEnabled + " | " + FLine + "User Mode: " + UserSelectedMode + " | User language: " + languageM + "");
			int nb_plugins = plugin_manager.getPlugins().length;
			String[] plugins_names = new String[nb_plugins];
			String sUserPluginDir = FileUtil.getUserFile("plugins").toString();
			if (!sUserPluginDir.endsWith(System.getProperty("file.separator"))) {
				sUserPluginDir += System.getProperty("file.separator");
			}

			String sAppPluginDir = FileUtil.getApplicationFile("plugins").toString();
			if (!sAppPluginDir.endsWith(System.getProperty("file.separator"))) {
				sAppPluginDir += System.getProperty("file.separator");
			}
			for(int i=0; i<nb_plugins; i++) {
					plugins_names[i] = plugin_manager.getPlugins()[i].getPluginName();
			}
			Arrays.sort(plugins_names);
			int begin = 0;
			int nb_per_line = 7;
		    int end = nb_per_line;
		    while(begin<nb_plugins) {
		    	String plugins = "";
			    for(int i=begin; i< end; i++) {
			    	boolean sKey = false;
					if (plugin_manager.getPlugins()[i].getPluginDirectoryName().length() > sUserPluginDir.length()
							&& plugin_manager.getPlugins()[i].getPluginDirectoryName().substring(0, sUserPluginDir.length()).equals(
									sUserPluginDir)) {
						sKey = true;
					} else if (plugin_manager.getPlugins()[i].getPluginDirectoryName().length() > sAppPluginDir.length()
							&& plugin_manager.getPlugins()[i].getPluginDirectoryName().substring(0, sAppPluginDir.length()).equals(
									sAppPluginDir)) {
						sKey = true;
					} 
					else  {
						sKey = false;
					}
					// FIXME or just comment out me... >_> 
					/*
					if (plugins_names[i] == "Client ID" || plugins_names[i] == "Azureus Update Support" || plugins_names[i] == "Azureus Updater Support Patcher" || plugins_names[i].startsWith("Core ") || plugins_names[i] == "Distributed DB" || plugins_names[i] == "Distributed Tracker") {
						sKey = false;
					}
					*/
					if (sKey == true) {
						plugins += plugins_names[i] + ", ";
					}
			    }
		    	begin += nb_per_line;
		    	end = Math.min(end + nb_per_line, nb_plugins);
		    	if (plugins != "") {
		    		this.sendMessage(sourceNick, "Plugins: " + plugins + "");
		    	}
		    }
		    this.sendMessage(sourceNick, "End of Info for " + userName + "");
		} else {
			this.sendMessage(sourceNick, "The User does not allow sending information");
		}
		
	} else {
	 		this.sendMessage(sourceNick, "You do not have permission to do that (" + channel + ")");
	}
	}
	catch (Exception e) {
		this.sendMessage(sourceNick, "Something went horribly wrong... Please notify another helper (Error: " + e.getMessage().toString() + ")"); 
	}
	super.onInfo(sourceNick, sourceLogin, sourceHostname, target);
}
}